//links of interest
//https://wltd.org/dmx-to-rgbw-led
//https://ptzoptics.com/wp-content/uploads/2014/09/PTZOptics_TCP_UDP_CGI_Control-1.pdf
//http://www.mathertel.de/Arduino/DMXSerial.aspx

//DOCUs
//zoom res. 1024 pqrs 4 16 16 0
//focus res. 4096 pqrs 0 16 16 16
//PIN A -> DMX pin 3
//PIN B -> DMX pin 2
//RO -> RX

#include <SoftwareSerial.h>
#include <DMXSerial.h>

//select the pins that will be used to connect to the TTL to RS485 converter or directly to the camera
SoftwareSerial SoftSerial(10, 11);

//this is the first channel of the camera fixture, to make it more flexible a 8 dip switch could be used to select the channel
unsigned int channel = 15; 
unsigned int zoom = 0;
unsigned int focus = 0;
//a delay must be added probably to avoid DMX overflow
unsigned int Delay = 30;

void setup() {    
  SoftSerial.begin(9600);
  DMXSerial.init(DMXReceiver);
}

void loop() {
  unsigned int newzoom = (DMXSerial.read(channel)*256) + (DMXSerial.read(channel+1));
  if (newzoom != zoom) {
    send(newzoom, 4, 16, 16, 1, 71);
    zoom = newzoom;
  }

  unsigned int newfocus = (DMXSerial.read(channel+2)*256) + (DMXSerial.read(channel+3));
  if (newfocus != focus) {
    send(newfocus, 1, 16, 16, 16, 72);
    focus = newfocus;
  }

  unsigned int focusmode = DMXSerial.read(channel+4);
  if (focusmode != 0) {
    if (focusmode > 127) {
      byte message[] = {0x81, 0x01, 0x04, 0x38, 0x02, 0xFF};
      for (int i = 0; i < 6; i++) {
        SoftSerial.write(message[i]);  
      }    
    }
    else {
      byte message[] = {0x81, 0x01, 0x04, 0x38, 0x03, 0xFF};
      for (int i = 0; i < 6; i++) {
        SoftSerial.write(message[i]);  
      } 
    }  
    delay(Delay);
  }
}

void send (int data, int bp, int bq, int br, int bs, int btype) {
  byte s = char(data%bs);
  int sdata= (data/bs);
  byte r = char(sdata%br);
  int rdata = sdata/br;
  byte q = char(rdata%bq);
  int qdata = rdata/bq;
  byte p = char(qdata%bp);

  byte type = char(btype);
  
  byte message[] = {0x81, 0x01, 0x04, type, p, q, r, s, 0xFF};
  for (int i = 0; i < 9; i++) {
    SoftSerial.write(message[i]);  
  } 
  
  delay(Delay);
}
